import json, asyncio, uuid, tempfile, time
from typing import List
from concurrent.futures import ThreadPoolExecutor

import boto3

try:
    from sagemaker.transformer import Transformer
except ImportError:
    Transformer = None

from animuz_core.utils import init_logger

LOGGER = init_logger(__name__)
LOGGER.setLevel("INFO")

class S3EmbeddingClient():
    def __init__(self) -> None:
        self.runtime = boto3.client("sagemaker-runtime")
        self.model_name = "huggingface-pytorch-inference-2024-05-28-06-25-32-617"
        self.endpoint_name = "bge-m3-serverless"
        self.s3_client = boto3.client("s3")
        self.s3_bucket_name = "animuz-datasource"
        self.s3_prefix = "batch_transform_datasource"
        self.executor = ThreadPoolExecutor(max_workers=1)

    def _get_embedding(self, text: str) -> dict:
        """
        Sends a POST request to the embedding server to get the embedding for the given text.
        
        Args:
            text (str): The text to get the embedding for.
        
        Returns:
            dict: The embedding of the given text in the form of a dictionary.
        """
        payload = json.dumps({"doc_id":1, "document_text": text}) + "\n"
        payload = payload.encode()
        start = time.time()
        response = self.runtime.invoke_endpoint(
            EndpointName=self.endpoint_name,
            ContentType="application/jsonlines",
            Accept="application/jsonlines",
            Body=payload
        )
        LOGGER.debug(json.dumps({
            "timestamp": time.strftime('[%Y-%b-%d %H:%M:%S %z]'),
            "message": "Sagemaker serverless embedding latency",
            "latency": str(time.time() - start),
            "exception": False
        }))
        data = response["Body"].read()
        data = json.loads(data.decode())
        data["embedding"] = data["dense_embeddings"] # keep forgetting to rename the keysss
        data["sparse_embedding"] = data["sparse_embeddings"]
        del data["dense_embeddings"], data["sparse_embeddings"]
        return data
    
    async def get_embedding(self, text: str) -> dict:
        try:
            loop = asyncio.get_event_loop()
            result = await loop.run_in_executor(self.executor, self._get_embedding, text)
        except Exception as e:
            LOGGER.critical(json.dumps({
                "timestamp": time.strftime('[%Y-%b-%d %H:%M:%S %z]'),
                "message": "ERROR Sagemaker serverless embedding",
                "latency": None,
                "exception": True
            }), exc_info=True)
            raise(e)
        return result

    def _batch_get_embedding(self, texts) -> List[dict]:
        start = time.time()
        #upload texts to s3
        object_prefix = self.s3_prefix + "/" + str(uuid.uuid4().hex) + "/" 

        with tempfile.NamedTemporaryFile() as tmp:
            data_jsonl = ""
            for i, text in enumerate(texts):
                record = {"doc_id": i + 1, "document_text": text}
                data_jsonl += json.dumps(record, ensure_ascii=False) + "\n"
            tmp.write(data_jsonl.strip().encode())
            tmp.seek(0)

            with open(tmp.name, 'rb') as f:
                self.s3_client.upload_fileobj(f, self.s3_bucket_name, object_prefix + "input.jsonl")

        #start transform job
        transformer = Transformer(model_name=self.model_name,
            instance_count=1,
            instance_type="ml.m5.2xlarge",
            strategy="MultiRecord",
            accept='application/jsonlines',  # Adjust based on your output format
            assemble_with='Line',  # Common for CSV input,
            output_path = f"s3://{self.s3_bucket_name}/{object_prefix}" 
            )
        
        transformer.transform(
            data=f"s3://{self.s3_bucket_name}/{object_prefix + 'input.jsonl'}",
            content_type='application/jsonlines',  # Specify the content type
            split_type='Line'  # Split type for JSONL input
        )

        transformer.wait()
        LOGGER.debug(json.dumps({
            "timestamp": time.strftime('[%Y-%b-%d %H:%M:%S %z]'),
            "message": "Sagemaker serverless batch embedding latency",
            "latency": str(time.time() - start),
            "exception": False
        }))
        #download result
        output = []
        with tempfile.NamedTemporaryFile() as tmp:
            self.s3_client.download_file(self.s3_bucket_name, f'{object_prefix}' + "input.jsonl.out", tmp.name)

            # Read and process the contents of the temporary file
            with open(tmp.name, 'r') as f:
                contents = f.read()

            for line in contents.splitlines():
                record = json.loads(line)
                record["embedding"] = record["dense_embeddings"]
                record["sparse_embedding"] = record["sparse_embeddings"]
                del record["dense_embeddings"], record["sparse_embeddings"]
                output.append(record)
        return output

    async def batch_get_embedding(self, texts):
        try:
            loop = asyncio.get_event_loop()
            result = await loop.run_in_executor(self.executor, self._batch_get_embedding, texts)
        except Exception as e:
            LOGGER.critical(json.dumps({
                "timestamp": time.strftime('[%Y-%b-%d %H:%M:%S %z]'),
                "message": "ERROR Sagemaker serverless batch embedding",
                "latency": None,
                "exception": True
            }), exc_info=True)
            raise(e)
        return result
    
if __name__ == "__main__":
    async def main():
        client = S3EmbeddingClient()
        text = ['<figure>  \n![](figures/0)  \n<!-- FigureContent="krungsri Capital Securities" -->  \n</figure>  \nA member of MUFG, a global financial group',
 '# Thai Union Group TU  \nEarnings Preview 1 April 2024',
 '## AGRO & FOODS  \nSummary  \nเรามีมุมมองเป็นกลาง คาด TU มีกำไรปกติใน 1Q24F ที่ 1,090 ลบ +35%y-y -7%q-q ภาพ y-y ชัดเจนว่าธุรกิจ petfood เด่นสุดด้วยยอดขายโต 15% GPM ฟื้นตัวจากสินค้าพรีเมียม เพิ่มขึ้นและต้นทุนทูน่าลดลง ธุรกิจอาหารทะเลกระป๋องโตน้อย ราคาทูน่าที่ลดลงทำให้ ราคาขายลด แต่ปริมาณเพิ่มขึ้นชดเชย GPM ดีขึ้นเล็กน้อย ต้นทุนทูน่าต่ำยังสะท้อนไม่ เต็มที่ในไตรมาสนี้ ธุรกิจอาหารทะเลแช่แข็งยอดขายอ่อนแต่ GPM ดีขึ้น Red Lobster ไม่ มีการบันทึกส่วนแบ่งแล้วใน 1Q24F ส่วน 2Q24F คาดกำไรปกติเพิ่ม y-y q-q กลุ่ม Petfood ยังโต กลุ่มอาหารทะเลกระป๋องดีขึ้นอีก กลุ่มอาหารทะเลแช่แข็ง ปริมาณและ GPM ดีขึ้น เพราะปรับโครงสร้างธุรกิจครบปีแล้ว เราแนะนำ Buy TU ด้วย TP 17.5 บาท (P/E 14 เท่า) บนแนวโน้มกำไรปกติปี 24F ฟื้นตัว +23% เพราะธุรกิจทั้งทูน่า/อาหารสัตว์เลี้ยง/อาหาร แช่แข็งจะมีความต้องการฟื้นตัวกลับมาในระดับปกติ หลังจากลูกค้าชะลอการซื้อในปี 23 เพื่อใช้สต๊อกเก่า แนวโน้มต้นทุนทูน่าลดลง',
 '# คาดกำไรปกติ 1Q24F ที่ 1,090 ลบ. +35%y-y -7%q-q  \nคาดกำไรสุทธิ 1,040 ลบ ใน 1Q24F เพิ่ม +2%y-y ฟื้นตัวจากขาดทุนใน 4Q23  \nเราคาดวันที่ 8 พ.ค. TU จะรายงานกำไรสุทธิ 1,040 ลบ.ใน 1Q24F เพิ่ม +2%y-y ฟื้นตัวจากขาดทุน -17,189 ลบ.ใน 4Q23 ส่วนกำไรปกติ 1,090 ลบ. ใน 1Q24F เพิ่ม +35%y-y ลด -7%q-q การเพิ่ม y-y เพราะธุรกิจ petfood มียอดขาย และ GPM เพิ่มจากตลาดฟื้นตัว ลูกค้าต้องการเพิ่มสต๊อก ขณะที่ธุรกิจ ทูน่ากระป๋องมีความต้องการเพิ่มแต่ราคาลด และ GPM ดีขึ้น ธุรกิจอาหารทะเลแช่แข็งทำการลดสินค้า ที่ไม่กำไรออก จึงส่งผลให้ยอดขายลดแต่ GPM ดีขึ้น ส่วนการลด q-q ตามฤดูกาลและธุรกิจทูน่า กระป๋องแบบ OEM ลดลงตามราคาทูน่าปรับลดราคาขายและมาร์จิ้นลง  \nRed Lobster ใน 1Q24F ไม่มีการรับรู้ส่วนแบ่งกำไร/ขาดทุนแล้ว ต่างจาก 1Q23 รับรู้ส่วนแบ่งกำไร 20 ลบ. และ 4Q23 รับรู้ขาดทุน -554 ลบ. หลัง TU ประกาศถอนการลงทุนใน RL  \nรายการพิเศษใน 1Q24F คาดขาดทุน FX -50 ลบ. เทียบกับ 1Q23 กำไร FX 215 ลบ. และ 4Q23 กำไร FX 67 ลบ  \nKey numbers : ยอดขายและ GPM เพิ่ม y-y แต่ลด q-q และไม่บันทึก Red lobster  \nยอดขายคาดที่ 33,898 ลบ. ใน 1Q24F เพิ่ม +4%y-y ลด -4%q-q ยอดขายเพิ่ม y-y โดย 1)ยอดขาย อาหารสัตว์เลี้ยงเพิ่ม +15%y-y ลูกค้ากลับมาซื้อและมีลูกค้า retailer ในสหรัฐเพิ่ม 2) ยอดขายอาหาร ทะเลกระป๋องลด y-y ด้วยราคาขายลดตามราคาทูน่าที่ลด -27%y-y ปริมาณขายเพิ่มขึ้นเพราะราคา จูงใจ 3)ยอดขายอาหารทะเลแช่แข็งลดลง -20%y-y เพราะอยู่ในช่วงปรับโครงสร้างตัดสินค้าที่ไม่ กำไรออกไป 4)ยอดขาย packaging และอาหารพร้อมรับประทานเพิ่มขึ้น ทั้งนี้ราคาทูน่าใน 1Q24 อยู่ที่US$1333/ตัน -27%y-y -13%q-q ยังช่วย GPM ธุรกิจทูน่ากระป๋องได้ไม่มาก แต่คาดว่าจะดีขึ้น ใน 2Q24F  \nGPM คาด 16.8% ใน 1Q24F vs 15.0%/17.8% ใน 1Q23/4Q23 การเพิ่ม y-y โดยธุรกิจอาหารแช่ แข็งปรับโครงสร้างธุรกิจลดสินค้าที่ไม่ทำกำไร ธุรกิจ petfood ปรับเพิ่มราคาขายและสินค้า premium เพิ่มขึ้น ธุรกิจอาหารกระป๋องมีต้นทุนทูน่าลดลง อย่างไรก็ตาม GPM รวมลด q-q ตามฤดูกาลที่ ธุรกิจ petfood มีการขายพรีเมี่ยมลดลง ธุรกิจอาหารทะเลแช่แข็งมีราคาขายลดตามกุ้งที่ราคาลดลงและพ รีเมี่ยมลดลง อย่างไรก็ตามเราคาดธุรกิจทูน่ากระป๋องมี GPM เพิ่ม เพราะปริมาณขายเพิ่มจึงมีต้นทุน คงที่ต่อหน่วยเพิ่มขึ้น  \nRed Lobster (RL) ไม่บันทึกส่วนแบ่งกำไร/ขาดทุนแล้ว ต่างจากบันทึกส่วนแบ่งกำไร(ขาดทุน) +20/ -554 ลบ.ใน 1Q23/4Q23  \n|||\n| - | - |\n| Rating | Buy |\n| Remains ||\n| TP 24F (14xP/E) | THB 17.50 |\n| Remains ||\n| Closing price | THB 14.50 |\n| Upside/Downside | +20.7% |  \nFundamental Investment Analyst on Securities  \nPloenjai Jirajarus Analyst Registration No. 17839 02 638 5000, 02 081 2000 Ploenjai.Jirajarus@krungsricapital.com  \n| Forecasts & Valuation |||||\n| 31 Dec (THB) | 23 | 24F | 25F | 26F |\n| - | - | - | - | - |\n| Revenue (mn) |||| 136,153 141,157 147,216 153,686    |\n| EBITDA (mn) |||| 10,698 11,781 12,024 12,429    |\n| Norm. Profit(mn) || 4,762 5,875  | 6,115 | 6,373 |\n| Net Profit (mn) || 13,933) 5,875  | 6,115 | 6,373 |\n| EPS (THB) | (3.15) | 1.26 | 1.31 | 1.37 |\n| EPS Growth (%) | \'314.29) | Up | 4.09 | 4.21 |\n| PER (x) | (4.60) | 11.49 | 11.04 | 10.59 |\n| BV/share | 12.59 | 13.34 | 14.03 | 14.74 |\n| P/BV (x) | 1.15 | 1.09 | 1.03 | 0.98 |\n| EV/EBITDA (x) :selected: | 11.13 | 9.22 | 8.98 | 8.63 |\n| DPS (THB) | 0.54 | 0.63 | 0.66 | 0.68 |\n| Dividend yield (%) | 3.72 | 4.35 | 4.53 | 4.72 |\n| ROE (%) | (20.01) | 9.73 | 9.60 | 9.52 |\n| Int. bearing D/E (x) :selected: | 1.03 | 0.98 | 0.94 | 0.89 |  \nSource: TU, KCS  \n| THBmn ||||| 1Q23 4Q23 1Q24F %y-y %q-q     |\n| - | - | - | - | - | - |\n| Sales ||||| 32,652 35,529 33,898 3.8% -4.6%     |\n| Gross profit | 4,917 |||| 6,327 5,695 15.8% -10.0%    |\n| % GPM ||| 15.06% 17.81% 16.80%   || 1.7% -1.0%  |\n| EBITDA | 1,725 | 3,124 ||| 2,597 50.5% -16.9%   |\n| % EBITDA ||||| 5.28% 8.79% 7.66% 2.4% -1.1%     |\n| Red Lobster | 20 | (554) || 0 -100%  | Up |\n| Norm profit | 806 | 1,177 ||| 1,090 35.3% -7.4%   |\n| % Norm | 2.47% | 3.31% ||| 3.22% 0.7% -0.1%   |\n| Net profit || 1,021 -17,189  || 1,040 1.8%  | Up |\n| FX gain | 215 | 67 | -50 | | |\n| Extra items || 1 -18,433  | O | | |  \nSource: TU, KCS  \n<!-- PageFooter="IRIS Division" -->  \n<!-- PageFooter="www.krungsricapital.com" -->  \n<!-- PageHeader="Krungsri Capital Securities | Thai Union Group" -->  \n<!-- PageHeader="1 April 2024" -->  \nส่วนแบ่งจากบริษัทย่อยคือ Avanti Feed และ Avanti Frozen กำไร +174 ลบ. ใน 1Q24F ลด -17%y-y เพิ่ม +13%q-q จาก +211/+154 ลบ.ใน 1Q23/4Q23 เพราะการส่งออกกุ้งและการเลี้ยงกุ้งในอินเดียลด y-y จากการแข่งราคาของเอกวาดอร์ แต่ภาพ q-q ดีขึ้นตามฤดูกาลมีการลงกุ้งในฤ ฤดูกาลใหม่',
 '## คาด 2Q24F กำไรปกติดีขึ้น y-y q-q รับรู้ต้นทุนทูน่าลดลง  \nคาดกำไรปกติใน 2Q24F ราว 1,300-1,400 ลบ เพิ่มขึ้น y-y q-q vs 1,275 /1,090 ลบ ใน 2Q23/1Q24F ภาพ y-y q-q 1) ธุรกิจ petfood ยอดขายและ GPM ดีขึ้น ปริมาณขายฟื้นตัว สินค้าพรีเมียมเพิ่ม และต้นทุนทูน่าลดลง 2) ธุรกิจอาหารทะเลกระป๋อง ปริมาณขายเพิ่มขึ้น GPM เพิ่มขึ้นจากต้นทุนทู น่าลดลง 3) ธุรกิจอาหารทะเลแช่แข็ง ปริมาณขายและ GPM ดีขึ้น โครงการลดสินค้าไม่กำไรทำให้ GPM ดีขึ้น และยอดขายฟื้นตัว',
 '# ราคาทูน่าเดือนมีนาคมอยู่ที่ US$1300/ton (-33%y-y คงที่m-m) คาดราคาทรงตัวระดับ US$1400-1700 ใน 2Q-4Q24  \nราคาทูน่าเดือนมีนาคม 2024 ที่ US$1300/ton ลดลง -33%y-y คงที่ m-m จาก US$1980 เดือนมีค.23 และ US$1300 เดือนกพ.24 ทำให้ราคาเฉลี่ย 1Q24F อยู่ที่ US$1333 ลด -27%y-y -13%q-q จาก US$1820 ใน 1Q23 และ US$1533 ใน 4Q23 ผู้บริหาร TU ให้ความเห็นว่าราคาทูน่าในปี 2024 จะ อยู่ระหว่าง US$1400-1700 ลดลงจากปี 2023 ที่เฉลี่ย US$1840 อย่างไรก็ตามเราคาดว่าราคาทูน่าจะทรงตัวถึงเพิ่มขึ้นเล็กน้อยในเดือนเมษายน เป็น ต้นไป เพราะหากราคาทูน่าต่ำมาก ชาวประมงก็ไม่อยากออกจับปลาที่ให้ผลตอบแทนน้อย',
 '## แนะนำ Buy ราคาเป้าหมาย 17.50 บาท (พีอี 14 เท่า= mean)  \nเราคงประมาณการกำไรปกติปี 2024/25F ไว้ที่เดิม เนื่องจาก preview GPM และ ยอดขายใน 1Q24F ถึงแม้จะต่ำกว่าคาดเดิมบ้าง แต่จะดีขึ้นใน 2Q24F โดยเฉพาะในส่วนของยอดขายและ GPM ของอาหารทะเลกระป๋อง  \nเราคงแนะนำ Buy ราคาเป้าหมาย 17.50 บาท เพราะแนวโน้มปี 2024F ผลการดำเนินงาน TU จะมีกำไรปกติเติบโต +23% เกิดจากธุรกิจทั้งทูน่ากระป๋อง Petfood จะมีความต้องการฟื้นตัวกลับมาในระดับปกติ หลังจากลูกค้าชะลอการซื้อในปี 2023 เพื่อใช้สต๊อกเก่า ประกอบกับคาดราคาทูน่าจะลดลงในปี 2024F ส่วนธุรกิจอาหารทะเลแช่แข็งคาดฟื้นตัวหลังปรับโครงสร้างธุรกิจ ยกเลิกสินค้ากลุ่มที่ไม่มีกำไรออกไป ทำให้ GPM ของธุรกิจอาหารแช่แข็งดีขึ้น นอกจากนี้การถอนการลงทุนใน Red Lobster ทำให้ TU สามารถfocus ในการขยายธุรกิจหลักของบริษัทได้ เราให้ TU เป็น Top pick ของกลุ่มเกษตรคู่ กับ GFPT เพราะการฟื้นตัวของ TU ที่ชัดเจนขึ้นในปี 2024F ขณะที่ GFPT เติบโตจากธุรกิจไก่ในประเทศและส่งออกไก่ที่ฟื้นตัว  \nFig 1. Earnings preview for 1Q24F and 2024F  \n| Profit and Loss (THB mn) |||||||||||\n| Year-end 31 Dec | 1Q23 | 4Q23 | 1Q24F | %y-y | %q-q | 2022 | 2023 | %y-y | 2024F | %y-y |\n| - | - | - | - | - | - | - | - | - | - | - |\n| Revenue | 32,652 | 35,529 | 33,898 | 3.82 || (4.59) 155,586  | 136,153 | (11.98) | 141,157 | 3.68 |\n| Gross Profit | 4,917 | 6,327 | 5,695 | 15.8 | (10.0) | 27,206 | 23,225 | (13.8) | 24,844 | 7.0 |\n| EBITDA | 1,725 | 3,124 | 2,597 | 50.5 | (16.9) | 11,971 | 10,698 | (8.6) | 11,781 | 10.1 |\n| Interest expense | (532) | (630) | (655) | 23.0 | 3.9 | (1,998) | -2,302 | 13.3 | -2,637 | 14.5 |\n| Corporate tax | 242 | 40 | (106) | (143.9) | (364.5) | 840 | 620 | (19.0) | -616 | (199.4) |\n| Forex gain (loss) | 215 | 67 | (50) | (123.3) | (174.6) | 800 | -236 | (131.0)  :unselected: || 0 (100.0) |\n| Other extra items | 1 | -18,433 || 0 (100.0)  | (100.0) | (665) | -18,459 | Down | O :unselected: | (100.0) |\n| Gn (Ls) from Avanti+Other | 211 | 154 | 174 | (17.5) | 13.0 | 600 | 679 | 16.6 | 700 | 3.1 |\n| Gn (Ls) from Red Lobster | 20 | (554) | O | (100) | (100) | (1,630) | -1,219 | Up | O :unselected: | Up |\n| Net Profit | 1,021 | -17,189 | 1,040 | 1.8 | (106.1) | 7,138 | -13,933 | Down | 5,875 | Up |\n| Norm. Profit | 806 | 1,177 | 1,090 | 35.3 | (7.4) | 7,003 | 4,762 | (28.1) | 5,875 | 23.4 |\n| EPS (THB) | 0.21 | (3.52) | 0.21 | 1.8 | (106.1) | 1.47 | (3.15) | Down | 1.26 | Up |\n| Gross Margin (%) | 15.06 | 17.81 | 16.80 | | | 17.49 | 17.06 | | 17.60 | |\n| SG&A/Sales (%) | 12.62 | 11.74 | 12.00 | | | 12.31 | 11.98 | | 12.00 | |\n| EBITDA Margin (%) | 5.28 | 8.79 | 7.66 | | | 7.69 | 7.86 | | 8.35 | |\n| Norm profit Margin(%) | 3.13 | (48.38) | 3.07 | | | 4.50 | 3.50 | | 4.16 | |\n| Net Margin (%) | 3.13 | (48.38) | 3.07 | | | 4.59 | (10.23) | | 4.16 | |\n| Current ratio (x) :selected: | 2.14 | 1.70 | 1.72 | | | 2.38 | 1.70 | | 1.72 | |\n| Interest Coverage (x) :selected: | 1.49 | 3.42 | 2.48 | | | 4.03 | 3.00 | | 3.00 | |\n| Debt to Equity (x) :selected: | 0.70 | 1.03 | 1.01 | | | 0.68 | 1.03 | | 0.98 | |\n| BV (THB/shr) | 16.88 | 12.59 | 12.82 | | | 16.90 | 12.59 | | 13.34 | |\n| ROE (%) | 5.75 | (102.55) | 5.93 | | | 10.20 | (20.01) | | 9.73 | |  \nSource: TU and KCS estimate  \n<!-- PageFooter="IRIS Division" -->  \n<!-- PageNumber="2" -->',
 '      คู่มือพนักงาน\n\n     บริษัท เค.พี.เทรดดิ้ง  กรุ๊ป  จำกัด\n\nRevised :1 พฤษภาคม 2562\n\nการจ้างงาน\n\nการจ้างพนักงานใหม่\n\n\tการจ้างพนักงานใหม่จะกระทำเมื่อบริษัทฯมีตำแหน่งและผู้สมัครมีคุณสมบัติ ความชำนาญซึ่งสามารถทำงานอย่างมีประสิทธิผลต่อบริษัท ฯเท่านั้น โดย\n\nผู้สมัครทุกคนจะได้รับการปฏิบัติอย่างยุติธรรมและเท่าเทียมกันโดยไม่คำนึงถึงเชื่อชาติ ศาสนา ผิว หรือเพศ\n\nการคัดเลือกผู้สมัคร จะพิจารณาจากคุณสมบัติของผู้สมัครเป็นเบื้องต้น โดยไม่คำนึงถึงความสัมพันธ์ส่วนตัว การลำเอียง หรือสินบนใดๆ\n\nประเภทของพนักงาน\n\nพนักงานรายเดือน คือพนักงานที่บริษัทฯตกลงจ้างให้เป็นการประจำโดยจ่ายค่าจ้างเป็นรายเดือนอันรวมถึงค่าจ้างในวันหยุดประจำสัปดาห์ วันหยุดตามประเพณี วันหยุดพักผ่อนประจำปี และวันลาตามเงื่อนไขที่กำหนดไว้\n\nพนักงานรายวัน คือพนักงานที่บริษัทฯตกลงจ้างไว้เป็นการประจำโดยจ่ายค่าจ้างเป็นรายวัน มีสิทธิ์ได้รับค่าจ้างเฉพาะวันที่มาทำงานและได้รับค่าจ้างในวันหยุดตามประเพณี และวันหยุดพักผ่อนประจำปี และวันลาตามเงื่อนไขที่กำหนดไว้\n\nการทดลองงาน\n\n\tพนักงานใหม่ทุกคนต้องอยู่ในระยะทดลองงานไม่เกิน 120 วัน ซึ่งในระยะเวลาดังกล่าว พนักงานจะได้รับการประเมินผลงาน และความเหมาะสมโดยเปรียบเทียบกับมาตราฐานที่กำหนดไว้ภายใน 90 วัน อย่างไรก็ตาม บริษัทฯอาจเลิกจ้างพนักงานเหล่านี้ออกเมื่อใดก็ได้ระหว่างการทดลองงานหากผลการปฏิบัติงานไม่เป็นที่\n\n-1-\n\nพอใจ หรือไม่มีความเหมาะสมกับงานในหน้าที่ โดยไม่จำเป็นต้องแจ้งล่วงหน้า และไม่จ่ายเงินชดเชยให้แต่ประการใดทั้งสิ้นและในระยะทดลองงาน ห้ามขาดงาน  และถ้าป่วยต้องมีใบรับรองแพทย์เป็นหลักฐาน\n\nการเลื่อนตำแหน่ง\n\n\tบริษัทฯมีนโยบายที่จะบรรจุพนักงานปัจจุบัน สำหรับตำแหน่งว่าง ก่อนที่จะพิจารณาจากผู้สมัครภายนอก บริษัทฯจะเป็นผู้ออกประกาศอย่างเป็นทางการในการเลื่อนตำแหน่งนั้นๆ',
 'เวลาทำงานปกติ วันทำงานปกติและวันหยุดประจำสัปดาห์\n\nชั่วโมงการทำงานต่อวัน\n\nพนักงานทุกท่านต้องทำงานวันละ 8 ชั่วโมง ไม่รวมเวลาพัก\n\nการกำหนดเวลาทำงาน\n\nวันทำงานปกติ วันจันทร์-วันเสาร์\n\nเวลาทำงานปกติวันจันทร์-วันศุกร์\t08.30-17.00 น.\n\nเวลาทำงานปกติวันเสาร์\t\t08.30-12.00 น.\n\nเวลาพักกลางวัน\t\t\t12.00-13.00 น.\n\nวันหยุดตามประเพณี\n\n\tบริษัทฯ กำหนดให้มีวันหยุดตามประเพณีอย่างน้อยปีละไม่น้อยกว่า 13 วันโดยรวม โดยบริษัทฯจะประกาศวันหยุดให้ทราบล่วงหน้าเป็นประจำในเดือนธันวาคมของแต่ละปี\n\n2-\n\nการบันทึกเวลาทำงาน\n\nพนักงานทุกท่านต้องบันทึกเวลาเข้า-ออก การบันทึกเวลาทำงานของพนักงานมีดังนี้\n\nพนักงานทุกท่านจะต้องบันทึกเวลาเข้า-ออกด้วยตนเองห้ามมีการบันทึกเวลาแทนกันโดยเด็ดขาด  ยกเว้นพนักงานบางระดับ บางตำแหน่งที่บริษัทฯ กำหนดให้ไม่ต้องบันทึกเวลาเข้า-ออก\n\nเมื่อมาทำงานแล้วต้องบันทึกเวลา(ตอกบัตรเข้า,ออก)งานทุกครั้ง\n\n ในกรณีที่พนักงานลืมตอกบัตร ต้องให้หัวหน้างานเซ็นต์อนุมัติรับรองในการทำงาน มิฉะนั้นจะถือว่าขาดงาน\n\nการบันทึกเวลาแทนกัน บริษัทฯจะถือเป็นความผิดที่มีโทษทางวินัยทั้งสองฝ่าย   และ จะได้รับการตักเตือน-พักงาน-ไล่ออก  ตามลำดับ\n\nการผิดวินัยการทำงานการเข้า-ออก\n\nการเข้า-ออก ประตูบริษัท หลังจากที่พนักงานตอกบัตรทำงานแล้ว\n\nหากมีความจำเป็น ต้องขออณุญาติหัวหน้างานก่อน เพื่อไม่ให้มีผลกระทบต่อธุรกิจ\n\nการผิดวินัยการตรงต่อเวลา-มาสาย\n\nพนักงานที่บันทึกเวลาปฎิบัติงานสายเกิน 8.30 น ถือว่าสาย\n\n\tสาย 1  ครั้ง/ 1 เดือน ไม่โดนหักเงิน\n\n\tสายครั้งที่ 2 จะถูกหักงินตามชั่วโมงเต็ม\n\n\tสายมากกว่า 3 ครั้ง/เดือน หัวหน้าตักเตือน และเสียประวัติในการประเมินผล\n\nการผิดวินัยการทำงาน-การแต่งเครื่องแบบ\n\nพนักงานต้องสวมเสื้อฟอร์มที่บริษัทฯกำหนดให้ในขณะปฏิบัติงาน มิฉะนั้นถือว่าไม่พร้อมที่จะเข้าทำงานได้ (ยกเว้นวันเสาร์)\n\n3-\n\nกางเกง/ กระโปรง ต้องสุภาพ ไม่ดูอุจาดตาที่จะทำให้เสื่อมเสียภาพลักษณ์ของบริษัท\n\nรองเท้า\n\nเพื่อความปลอดภัยแก่ตัวพนักงานเอง ขณะปฏิบัติงานกับเครื่องจักรหรือขนสายไฟ พนักงานต้องสวมใส่รองเท้าหุ้มนิ้วเท้าให้มิดชิด ห้ามสวมรองเท้าชำรุดเข้าทำงาน\n\nเพื่อภาพลักษณ์ในการติดต่อลูกค้า พนักงานควรใส่รองเท้าสุภาพ ห้ามสวมรองเท้าแตะสกปรก หรือถอดรองเท้าพบปะลูกค้า หากฝ่าฝืนให้หยุดการทำงาน',
 'วินัยและความประพฤติ\n\n\tพนักงานแต่ละท่านได้รับการคาดหมายให้ปฏิบัติหน้าที่ของตนตามความสามารถอย่างดีที่สุด และต้องประพฤติปฏิบัติให้เหมาะสมเพื่อชื่อเสียงอันดีของ\n\nบริษัทฯ ทั้งในและนอกสถานที่ พนักงานทุกท่านต้องเคารพ เชื่อฟัง คำสั่งอันชอบด้วยกฏหมายของหัวหน้างาน และต้องปฏิบัติตามกฎ ตลอดจนระเบียบต่างๆ ซึ่งกำหนดไว้\n\nการดำเนินทางวินัย\n\n\tวัตถุประสงค์ของการดำเนินการทางวินัย คือการแก้ไข ปรับปรุงทัศนคติ นิสัย หรือความประพฤติของพนักงาน  บริษัท ฯ จะดำเนินการทางวินัยต่อพนักงานในกรณีของการไร้ประสิทธิภาพหรือฝ่าฝืนกฏ ระเบียบ หรือขัดขืนคำสั่งอันชอบด้วยกฏหมายดังนี้\n\nการไร้ประสิทธิภาพ\n\nในกรกณีที่บริษัทฯ มีความเห็นว่าพนักงานไม่สามารถปฏิบัติหน้าที่ของตนเองอย่างมีประสิทธิภาพ ทางบริษัทฯ อาจเลิกจ้างพนักงานผู้นั้นเมื่อใด\n\n4-\n\nก็ได้ โดยการแจ้งให้ทราบล่างหน้าอย่างน้อย 1 เดือน ภายหลังออกหนังสือเตือนในเรื่องดังกล่าว\n\nการฝ่าฝืนกฏระเบียบ บริษัท หรือขัดขืนคำสั่งอันชอบด้วยกฏหมาย\n\nการดำเนินการทางวินัยจะรุนแรงระดับใดขึ้นอยู่กับเจตนา ลักษณะของการ\n\nกระทำและผลจากการกระทำนั้นๆ โดยไม่จำเป็นต้องเรียงลำดับ และระดับต่างๆ ที่บริษัทฯจะเลือกใช้ดังนี้\n\nก. การเตือนด้วยวาจา เป็นการดำเนินการทางวินัยที่มีลักษณะเบาที่สุด\n\n                    จำนวน  …2...ครั้ง\n\n \tข. การเตือนเป็นลายลักษณ์อักษร เป็นการดำเนินการทางวินัย ในกรณีที่มี\n\nการละเมิดกฏ ข้อบังคับที่สำคัญหรือร้ายแรงหรือการกระทำความผิดวินัย จำนวนรวมครั้งที่ออกใบเตือน..2..ครั้ง   อายุความ  1  ปีนับจากวันเตือนครั้งแรก ประวัติการออกใบเตือนนี้จะนำไปพิจารณาโบนัสและเงินเดือนใน\n\nปีต่อๆไปด้วย และถ้าออกใบเตือนครั้งที่3  จะให้  พักงาน  ทันทีเป็นเวลา\n\n1 อาทิตย์ โดยไม่จ่ายค่าจ้าง\n\nค.การพักงาน เป็นการดำเนินการทางวินัยขั้นรุนแรง แต่ไม่ถึงขั้นไล่ออก ซึ่งเป็นการสั่งให้หยุดงานชั่วคราว โดยไม่จ่ายเงิน   แล้วทำผิดอีก  จะถูกให้ปลดออกโดยทันที',
 'ง. การไล่ออก เป็นการดำเนินการทางวินัยขั้นรุนแรงที่สุด โดยบริษัทฯจะไม่จ่ายเงินชดเชยใดๆ ทั้งสิ้น\n\nความผิดที่จะออกใบเตือน\n\nการทะเลาะวิวาทในสถานที่ทำการของบริษัทฯ\n\nการกระทำเจตนาทำให้ทรัพย์สินของบริษัฯ เสียหาย\n\nการกระทำโดยก่อให้เกิดอันตรายต่อตนเองและผู้อื่น\n\nการเข้าทำงานในสภาพมึนเมาจากพิษสุราหรือสิ่งเสพติดทุกชนิด\n\n5-\n\nการครอบครองหรือดื่มสุราภายในบริษัทฯโดยไม่ได้รับอนุญาต\n\nการตอกบัตรเข้างานแทนกัน\n\nการละเลยหน้าที่ ของตนเอง (อ้างอิงตามใบ Job Description)\n\nการกระทำที่ทำให้บริษัทเสียหายเช่น การกล่าวร้ายบริษัทในสื่อ Social Networkที่ทำให้บริษัทเสียภาพลักษณ์ และชื่อเสียง\n\nการตักเตือน  การตักเตือนด้วยวาจา  และการออกใบเตือน   ทุกครั้งจะต้องมีการบันทึกลงสมุด พร้อมลายเซ็นต์ ของหัวหน้าแผนก และ ฝ่ายบุคคล   \n\nความผิดที่ไล่จะออกทันที\n\n        1.การขโมย ยักยอก ทรัพย์สินของบริษัทฯ หรือของพนักงานอื่นๆ\n\n        2.การครอบครอง พกพา หรือขาย หรือเสพสิ่งเสพติดภายในบริษัทฯ\n\n        3.การละทิ้งหน้าที่เป็นเวลา 3 วัน ทำงานติดต่อกันไม่ว่าจะมีวันหยุดคั่นหรือไม่    ก็ตามโดยไม่มีเหตุอันควร\n\n        4.ฝ่าฝืนกฏข้อบังคับด้านความปลอดภัยซึ่งอาจก่อให้เกิดความเสียหายอย่างร้ายแรง\n\n       5.เจตนาปลอมแปลงหรือแก้ไขเอกสารสำคัญที่เกี่ยวข้องกับการเงินต่างๆ\n\n    **  เรื่องการเคารพต่อหัวหน้างาน ต้องเชื่อฟังหัวหน้างานและฝ่ายบุคคล ถ้าไม่เชื่อฟังจะโดนตักเตือน ทันที\n\n     **พฤติกรรมการทำงาน ห้ามใช้อารมณ์กับเพื่อนร่วมงาน เมื่อมาทำงานแล้วต้องปรับอารมณ์ให้ดี เพื่อทำงาน\n\n         ทั้ง 2 ข้อนี้ จะมีกฎระเบียบดังนี้\n\n         ครั้งแรก ตักเตือนดัวยวาจา\n\n         ครั้งที่ 2 โดนใบเตือนทันที\n\n         ครั้งที่ 3 สั่งพักงานและ ตัดเงิน\n\n         ครั้งที่ 4  ให้ออก\n\n6-',
 '   ***กรณีทะเลาะวิวาทรุนแรง  ***บริษัทฯสามารถใล่ออกจากงานทันที  \n\nเงินเดือน\n\nวันและสถานที่จ่ายค่าจ้าง\n\n\tบริษัทฯ จะจ่ายค่าจ้างแก่พนักงานเดือนละ 1 ครั้งทุกสิ้นเดือน โดยจ่ายเป็นเงินสด ณ ที่ทำการของบริษัทฯสำหรับพนักงานที่ยังไม่ครบทดลองงาน  และพนักงานที่ครบทดลองงานแล้วจะจ่ายผ่านธนาคารกรุงเทพฯ\n\nการปรับค่าจ้างและเงินเดือน\n\n\tบริษัทฯได้กำหนดให้มีการปรับค่าจ้างและเงินเดือน 3 ลักษณะ ดังนี้คือ\n\nการปรับค่าจ้างตามกฏหมายบังคับ\n\nการปรับประจำปี โดยปกติการปรับเงินเดือนลักษณะนี้จะกระทำปีละ 1 ครั้งโดยมีผลจากวันที่ 1 มกราคม ของแต่ละปี อย่างไรก็ตาม การปรับเงินเดือนลักษณะนี่ มิใช่  การปรับโดยอัตโนมัติ แต่จะพิจารณาจากผลการปฏิบัติงานของพนักงาน (ผลการทำงานโดยทั่วไป ความประพฤติ ผลสำเร็จจากการทำงาน)\n\nการปรับเมื่อเลื่อนตำแหน่ง การปรับเงินเดือนลักษณะนี้จะกระทำเมื่อพนักงานได้รับการเลื่อนตำแหน่งที่สูงขึ้นและสามารถปฏิบัติหน้าที่ในตำแหน่งใหม่ได้เป็นอย่างดี หรือในกรณีที่พนักงานได้รับการประเมิณผลการปฏิบัติงานในระดับยอดเยี่ยม\n\nค่าล่วงเวลาและค่าทำงานในวันหยุด\n\n\tการทำงานล่วงเวลา คือการทำงานที่ได้รับอนุมัติให้ทำเกินเวลาทำงานปกติของพนักงานในวันทำงาน หรือในวันหยุดประจำสัปดาห์ หรือในวันหยุดตามประเพณี\n\nที่บริษัทฯประกาศ     หัวหน้างาน ต้องเป็นผู้อนุมัติเป็นลายลักษณ์อักษรก่อนที่จะ\n\n7-\n\nอนุญาตให้มีการทำงานล่วงเวลา  หรือทำงานในวันหยุดยกเว้นในกรณีฉุกเฉิน',
 'หลักเกณฑ์การจ่ายค่าล่วงเวลาและค่าทำงานในวันหยุด\n\nการทำงานล่วงเวลาให้แยกระหว่างแผนกขนส่ง กับ  แผนกสต๊อกสินค้า\n\nพนักงานที่ได้รับมอบหมายให้ทำงานล่วงเวลาในวันทำงานปกติ จะได้รับค่าล่วงเวลาในอัตรา 1.5 เท่าของอัตราค่าจ้างต่อชั่วโมงในเวลาทำงานปกติ    แผนกขนส่งเริ่มจากเวลา 17.00  แผนกสต๊อคเริ่มจากเวลา17.30 น. เป็นต้นไป (สต๊อคต้องลงเวลาทำงานจริงเท่านั้น)\n\nพนักงานที่ได้รับมอบหมายให้ทำงานล่วงเวลาในการทำงานวันหยุด คือวันหยุดประจำสัปดาห์ หรือวันหยุดตามประเพณีของบริษัทฯ (หลังเวลา 17.30น.) จะได้รับค่าล่วงเวลาในอัตรา 2 เท่าของอัตราค่าจ้างต่อชั่วโมงในเวลาทำงานปกติ\n\nพนักงานที่ทำงานล่วงเวลาต้องบันทึกการทำงานล่วงเวลาในแบบฟอร์มที่บริษัทฯ กำหนดไว้เพิ่มเติม และให้หัวหน้างานเซ็นรับรองการทำงานก่อนส่งให้ฝ่ายบุคคล\n\nการทำงานในวันหยุดประจำสัปดาห์และวันหยุดตามประเพณี\n\nพนักงานรายเดือนซึ่งได้รับค่าจ้างในวันหยุดอยู่แล้ว หากได้รับมอบหมายให้ทำงานวันหยุดประจำสัปดาห์หรือวันหยุดตามประเพณีจะได้รับค่าจ้างเพิ่มไม่เกิน 1 เท่า ของค่าจ้างปกติ\n\nพนักงานรายวัน ซึ่งได้รับมอบหมายให้มาปฏิบัติงานในวันหยุดประจำประจำสัปดาห์จะได้รับค่าจ้างเพิ่มไม่เกิน 2 เท่าของอัตราค่าจ้างปกติ\n\nพนักงานรายวัน ซึ่งได้รับมอบหมายให้มาปฏิบัติงานในวันหยุดตามประเพณี จะได้รับค่าจ้างเพิ่มไม่เกิน 1 เท่าของอัตราค่าจ้างปกติ เพราะพนักงานได้รับค่าจ้างในวันหยุดตามประเพณีอยู่แล้ว\n\n8-\n\nการลาหยุด\n\nวันลา\n\n\tบริษัทฯอนุญาตให้พนักงานลาหยุดได้ตามประเภทของการลาต่างๆ กัน พนักงานต้องมีความรับผิดชอบต่อบริษัทฯ และเพื่อนร่วมงาน โดยลาเฉพาะเหตุที่จำเป็นเท่านั้น ซึ่งการหยุดงานของท่านนั้น อาจสร้างความยุ่งยาก หรือความลำบากให้กับเพื่อนร่วมงานได้ พนักงานต้องกรอก “ใบลา” สำหรับการลาทุกประเภท และมอบต่อฝ่ายบุคคลเพื่ออนุมัติทุกครั้งที่ลาหยุด ในบางกรณีพนักงานต้องแนบเอกสารในการลาด้วย',
 '\tพนักงานทุกท่านซึ่งปฏิบัติงานมาครบ 1ปีจะได้รับวันหยุดประจำปี ดังนี้\n\nวันหยุดพักผ่อนประจำปี\n\nพนักงานทุกท่านจะได้รับวันหยุดพักผ่อนประจำปีท่านละ  10 วัน สำหรับพนักงานใหม่ที่ผ่านการทดลองงานแล้วก็ให้ลดลงตามส่วน\n\nถ้าลาเกินสิทธิ 10 วัน  ถือว่าเป็นการหยุดงานไม่รับค่าจ้าง หักตามจำนวนวันที่หยุดเกินตามค่าจ้างรายวัน\n\nการลาป่วย\n\nพนักงานมีสิทธิลาป่วยได้ตามจำนวนที่ป่วยจริง บริษัทฯจะจ่ายค่าจ้างให้ไม่เกิน 30 วันทำงาน แต่ต้องมีใบรับรองแพทย์จากโรงพยาบาลที่ทำประกันสังคมเท่านั้น (ไม่ขอรับใบรับรองแพทย์จากทางคลีนิค) และให้โทรลาแจ้งหัวหน้างานได้  และต้องส่งใบลาพร้อมแนบใบรับรองแพทย์ในวันที่กลับมาทำงานให้หัวหน้างานเซ็นต์รับรอง และนำส่งทางบัญชีบุคคล มิฉะนั้นจะถือเป็นการขาดงาน\n\nกรณีการไปพบแพทย์ตามนัดถือเป็นการลาป่วย(ให้แนบใบนัด)\n\nพนักงานที่แจ้งลาป่วยเป็นเท็จจะต้องถูกดำเนินการทางวินัยตามระเบียบ\n\n  ของบริษัทฯ ด้วย\n\n9-\n\nวันลาเมตตาจิตที่ได้รับค่าจ้างเท่ากับวันทำงานปกติ\n\nไม่ถือเป็นการลากิจหรือลาป่วย มีดังต่อไปนี้\n\n3.1 วันลาถึงแก่กรรมของครอบครัว\n\nบิดา มารดา คู่สมรสและบุตรที่ถูกต้องตามกฏหมาย\n\nบริษัทให้สิทธิ์ลาหยุดได้ 7 วัน โดยต้องแนบใบมรณบัตรด้วย\n\n3.2 วันลาคลอด\n\nสตรีที่มีครรภ์ ลาก่อนและหลังคลอดได้ไม่เกิน 90 วัน รวมวันนัดหมอตรวจครรภ์พนักงานจะได้รับค่าจ้างระหว่างลาคลอดไม่เกิน 45 วัน\n\nสามี มีสิทธิลาเมื่อภรรยาคลอดบุตรได้ 5 วัน\n\n3.3 วันลาอุปสมบท\n\nเป็นการลาได้เพียงครั้งเดียว ไม่เกิน 15 วัน (นับรวมวันหยุด) โดยพนักงานต้องทำงานครบ 1 ปี ถึงจะได้รับสิทธิ์ได้\n\nการลากิจ (ไม่ได้รับค่าจ้าง)\n\nการลากิจกรณีต่างๆ พนักงานต้องยื่นใบลาก่อนล่วงหน้า 1 วัน (กรณีมีเหตุพิเศษไม่อาจส่งใบลาล่วงหน้าได้ ให้โทรศัพท์แจ้งหัวหน้างาน และจัดส่งใบลาทันทีในวันแรกที่มาปฏิบัติงานให้กับหัวหน้างานนำส่งให้บัญชีบุคคล)\n\nเขียนใบลาและส่งใบลาในกรณีต่างๆ\n\nถือเป็นหน้าที่ของตัวพนักงานที่จะต้องส่งใบลาล่วงหน้า ให้กับหัวหน้างาน อนุญาติ รับรองการลาว่าสามารถจัดการได้ไม่กระทบก่อความเสียหายให้บริษัท นำใบลาที่หัวหน้าอนุมัติส่งให้ฝ่ายบัญชีบุคคล หากไม่ปฎิบัติตามจะ',
 'ถือว่าเป็นการผิดวินัยการลา บกพร่องในการทำงาน เป็นการขาดงานละทิ้งหน้าที่โดยไม่มีเหตุอันสมควร\n\n10-\n\nพนักงานที่ไม่มีสิทธิได้รับเงินค่าล่วงเวลาหรือค่าทำงานในวันหยุด\n\nพนักงานที่สมัครใจทำงานล่วงเวลา หรือทำงานในวันหยุด โดยหัวหน้างานมิได้มอบหมายและอนุญาต\n\nพนักงานที่หัวหน้างานไม่ได้เซ็นรับรองการทำงานล่วงเวลาหรือทำงานวันหยุดในแบบฟอร์มการบันทึก\n\nการพ้นสภาพการเป็นพนักงาน\n\n\t1.  การลาออก\n\n\tพนักงานทุกคนซึ่งครบระยะทดลองงานแล้ว และประสงค์จะลาออกต้องแจ้ง\tให้บริษัทฯ ทราบล่วงหน้าเป็นลายลักษณ์อักษร อย่างน้อย 1 เดือน\n\n\t2.  การปลดออก\n\n\tหมายถึงการที่บริษัทฯ ให้พนักงานพ้นสภาพจากการเป็นพนักงานด้วยสาเหตุดังต่อไปนี้\n\nการลดอัตรากำลัง พนักงานจะถูกเลิกจ้างเพื่อเป็นการลดอัตรากำลัง ในกรณีที่บริษัทฯมีความจำเป็นในสถานการณ์ต่างๆ การเลิกจ้าง เพื่อลดอัตรากำลังนี้มิได้เกิดจากความผิดของพนักงานแต่เป็นผลจากการเปลี่ยนแปลงทางด้านเทคโนโลยีการจัดรูปแบบองค์การใหม่ ความตกต่ำทางเศรษฐกิจ หรือการเงินของบริษัทฯ เป็นต้นโดยบริษัทจะจ่ายค่าชดเชยรายได้ให้\n\nถูกสั่งพักงาน ครบ 1  ครั้งแล้ว  และยังทำผิดซ้ำอีก\n\n\t3.  การไล่ออก\n\n\t เนื่องจากพนักงานกระทำผิดระเบียบ วินัยร้ายแรง ซึ่งมีเหตุผลพอเพียงที่จะไล่ออกได้ทันทีที่กระทำผิด เป็นผลจากการลงโทษทางวินัยตามขั้นตอนที่ระบุไว้ในกฏระเบียบของบริษัทฯพนักงานผู้นั้นมีความผิด ดังต่อไปนี้\n\n11-\n\n1. ทุจริตต่อหน้าที่หรือกระทำความผิดอาญาโดยเจตนาแก่นายจ้าง\n\n2 .จงใจทำให้นายจ้างได้รับความเสียหาย\n\n3.ประมาทเลินเล่อเป็นเหตุให้นายจ้างได้รับความเสียหายอย่างร้ายแรง\n\n4. ฝ่าฝืนข้อบังคับเกี่ยวกับการทำงาน ระเบียบ หรือคำสั่งของนายจ้างอัน ชอบด้วยกฎหมายและเป็นธรรม และนายจ้างได้ตักเตือนเป็นหนังสือแล้ว\n\n เว้นแต่กรณีที่ร้ายแรงนายจ้างไม่จำเป็นต้องตักเตือน ทั้งนี้ หนังสือเตือน',
 'ให้มีผลบังคับได้ไม่เกิน 1 ปี นับแต่วันที่ลูกจ้างได้กระทำผิด   \n\n5.ละทิ้งหน้าที่เป็นเวลา 3 วันทำงานติดต่อกันโดยไม่มีเหตุผลที่สมควร\n\n6.ได้รับโทษจำคุกตามคำพิพากษาถึงที่สุดให้จำคุก\n\n\t4.  การเกษียณอายุ\n\n\tอายุเกษียณตามปกติของพนักงานทุกคน คือ 55 ปีบริบูรณ์ อย่างไรก็ตามพนักงานอาจสมัครใจของเกษียณอายุก่อนกำหนดได้ หากพนักงานทำงานมาแล้วไม่น้อยกว่า 15ปี และมีอายุไม่น้อยกว่า 55 ปี บริษัทฯของสงวนสิทธิ์ที่จะจ้างพนักงานที่ครบเกษียณแล้วหรือปฏิเสธคำขอสมัครใจเกษียณอายุก่อนกำหนดก็ได้\n\nแนวปฏิบัติในการพ้นสภาพการเป็นพนักงาน\n\nการเลิกจ้างมีดังนี้\n\nคืนทรัพย์สินทุกอย่างของบริษัทฯ ทรัพย์สินต่างๆนั้น รวมถึงทรัพย์สินที่เป็นรูปธรรมเช่น คอมพิวเตอร์ เครื่องมือ อุปกรณ์ต่างๆ และทรัพย์สินที่ไม่เป็นตัวตน เช่นเอกสารโต้ตอบต่างๆ เป็นต้น ข้อที่ควรเอาใจใส่ในการคืนทรัพย์สินต่างๆ นั้นต้องให้บริษัทฯ รับทราบด้วย\n\n12-\n\nสวัสดิการและอื่นๆ\n\nโบนัสประจำปี\n\n                บริษัทฯมีโบนัสให้กับพนักงานทุกปีเพื่อเป็นขวัญและกำลังใจให้กับพนักงาน\n\nกองทุนประกันสังคม\n\n\tกองทุนประกันสังคมได้รับการจัดตั้งโดยกระทรวงแรงงาน และสวัสดิการทางสังคม เพื่อช่วยเหลือทางการเงินด้านค่ารักษาพยาบาล หรือการขาดรายได้ อันเป็นผลมาจากการเจ็บป่วย อุบัติเหตุ ทุพพลภาพ หรือเสียชีวิต ซึ่งไม่เกี่ยวกับหน้าที่การงานของพนักงาน และรวมถึงการคลอดบุตรด้วย                                                        \n\n\tกองทุนนี้เป็นการจ่ายเงินสมทบร่วมกันในอัตราเท่ากันทุกเดือนระหว่าง\n\nบริษัทฯ และพนักงาน อัตราการจ่ายสมทบปัจจุบันคือ ร้อยละ 5 ซึ่งคำนวณจากเงินเดือนของพนักงานและส่วนบริษัทฯ จะต้องจ่ายสมทบอีก 5 % ของยอดเงินหักทั้งหมด  \n\nการตรวจสุขภาพประจำปี\n\n                บริษัทฯได้จัดให้มีการตรวจสุขภาพประจำปีให้กับพนักงานทุกคน',
 'เครื่องแบบ\n\n\tเพื่อเป็นการช่วยเหลือค่าใช้จ่ายในการแต่งกายของพนักงานและเพื่อให้การแต่งกายเป็นไปในลักษณะเดียวกัน ทางบริษัทฯ จึงเป็นผู้จัดสรรเสื้อฟอร์มให้กับพนักงานคนละ 4 ตัว ต่อปี\n\nการจัดงานเลี้ยงสังสรรค์ปีใหม่\n\n                บริษัทฯได้จัดให้มีงานเลี้ยงสังสรรค์ปีใหม่ให้กับพนักงานทุกคน\n\n13-\n\nความสำคัญของพนักงานที่มีต่อบริษัทฯ\n\n\tวัตถุประสงค์สำคัญในการบริหารงานบุคคลของบริษัทฯ คือการว่าจ้าง การจูงใจ การรักษาไว้ และการพัฒนาคุณภาพ ของพนักงานทุกคนเพื่อให้บรรลุเป้าหมายของบริษัทฯ อย่างมีประสิทธิภาพ โดยจ่ายค่าตอบแทนและผลประโยชน์ที่เป็นธรรมแก่พนักงาน ทั้งนี้ขึ้นอยู่กับขอบเขตหน้าที่และความรับผิดชอบของแต่ละตำแหน่ง\n\nนโยบายทั่วไปในการบริหารงานบุคคลของบริษัทฯมีดังนี้\n\nการรักษาและส่งเสริมให้เกิดความเข้าใจและความสัมพันธ์อันดีระหว่างพนักงานกับผู้บริหารตลอดจนระหว่างพนักงานด้วยกัน\n\nจัดให้มีการประเมินผลอย่างมีประสิทธิภาพ และเป็นธรรมแก่พนักงาน โดยการเปรียบเทียบกับหน้าที่ความรับผิดชอบ และเป้าหมายที่กำหนดไว้ล่วงหน้าอย่างน้อยปีละครั้ง\n\nปรับปรุงและรักษาสภาพการทำงาน และเครื่องมือ อุปกรณ์ต่างๆ ให้ถูกต้องตามมาตรฐานความปลอดภัยและสุขอนามัย\n\nส่งเสริมให้พนักงานทุกท่านได้รับการพิจารณาเลื่อนตำแหน่งโดยเท่าเทียมกัน\n\nพิจารณาแก้ไขข้อร้องทุกข์ของพนักงานโดยไม่ชักช้า\n\nสำรวจทัศนคติของพนักงาน ที่มี ต่อนโยบาย ข้อบังคับ และสภาพการทำงานของบริษัทฯอย่างสม่ำเสมอ\n\n               กฎการใช้มือถือของบริษัทฯ\n\n           \t1.ห้ามใช้คุยเรื่องส่วนตัว\n\n \t2.ห้ามผู้ไม่มีกิจธุระ เอาโทรศัพท์มือถือบริษัทฯ  กลับบ้าน นอกจากผู้นั้นได้รับ อนุญาตแล้วเท่านั้น    \n\n14-',
 'ความคาดหวังของบริษัทฯที่มีต่อพนักงาน\n\nพนักงานทุกคนต้องปฏิบัติตามกฏ และข้อบังคับในการทำงาน ซึ่งชอบด้วยกฏหมายที่บริษัทฯ กำหนด และเห็นชอบร่วมกันแล้ว ตลอดจนต้องยอมรับสิทธิของฝ่ายบริหารในการดำเนินธุรกิจให้เป็นไปตามนโยบาย เป้าหมาย และมาตรฐานซึ่งได้กำหนดไว้โดยไม่กระทบต่อสิทธิและผลประโยชน์ของพนักงาน\n\nพนักงานแต่ละคนต้องทราบหน้าที่และความรับผิดชอบตลอดจนเงื่อนไขการว่าจ้างของตนเองและต้องปฏิบัติตามอย่างเคร่งครัด\n\nพนักงานทุกคนต้องทำงานร่วมกันอย่างเต็มกำลังความสามารถ และระลึกเสมอว่าความสัมพันธ์ในการทำงาน ทัศนคติ และความประพฤติของพนักงานทุกคนจะก่อให้เกิดผลกระทบต่อเพื่อนร่วมงาน และบริษัทฯซึ่งจะรวมไปถึงผลผลิต กำไร ค่าจ้าง เงินได้อื่นๆ ตลอดจนความเป็นอยู่ของ\n\nพนักงานด้วย ดังนั้นพนักงานต้องให้ความร่วมมือสนับสนุนฝ่ายบริหารในการพัฒนาปรับปรุงการผลิต ประหยัดค่าใช้จ่าย และปกป้องผลประโยชน์ของบริษัทฯ\n\nพนักงานทุกคนต้องรับผิดชอบในการดูแลรักษา อาคาร สถานที่ เครื่องใช้ และอุปกรณ์ต่างๆ ให้อยู่นในสภาพที่ปลอดภัย และถูกสุขอนามัย พนักงานต้องรายงานสิ่งที่อาจก่อให้เกิด การบาดเจ็บหรือมีสภาพที่ไม่น่าปลอดภัยแก่หัวหน้างานทราบทันที นอกจากนั้นพนักงานต้องใช้อุปกรณ์ป้องกันอันตรายส่วนบุคคลที่บริษัทฯ จัดหาให้ทุกครั้งสำหรับงานที่จำเป็นต้องใช้อุปกรณ์ดังกล่าว ห้ามนำออกนอกบริษัทฯ ถ้ามีเหตุจำเป็นต้องใช้ให้ขอ\n\nอนุญาติก่อน\n\n15-\n\nพนักงานทุกคนต้องพัฒนาความรู้และความสามารถในการปฏิบัติงานเพื่อให้ทันกับการเปลี่ยนแปลงต่างๆ ตลอดเวลา  ในการปฎิบัติงาน หรือว่ากระบวนการทำงานใดที่เยินเย่อไม่จำเป็น มีจุดใดที่สามารถนำเทคโนโลยีมาใช้แทนได้บ้างควรศึกษาเพิ่มเติม\n\nพนักงานต้องไม่เปิดเผยข้อมูลความลับที่รับทราบ จากการปฏิบัติงานในหน้าที่ของตนเอง ยกเว้นในกรณีที่ได้รับอนุญาตแล้ว และห้ามนำข้อมูลดังกล่าวไปแสวงหาผลประโยชน์ด้วยตนเอง หรือผ่านบุคคลที่สามโดยผิดจรรยาบรรณ หรือผิดกฏหมาย\n\nพนักงานต้องหลีกเลี่ยงการกระทำใดๆ ที่ขัดกับผลประโยชน์ของบริษัทฯ หรือที่ก่อให้เกิดผลเสียต่อหน้าที่การงานของตนเอง\n\nพนักงานจะต้องละเว้น การกระทำที่ผิดกฏระเบียบ หรือจรรยาบรรณของ',
 'บริษัทฯ ทั้งทางตรงและทางอ้อม\n\nมีความเข้าใจทางด้านความปลอดภัยในการปฏิบัติงานทุกขั้นตอน\n\n16-\n\nสารบัญ\n\n\t\t\t\t\t   หน้า\n\n\tการจ้างงาน\t\t\t\t\t1\n\n\tวินัยและความประพฤติ\t\t\t\t4\n\n\tเงินเดือน\t\t\t\t\t7\n\n\tการลาหยุด\t\t\t\t\t9\n\n\tการพ้นสภาพการเป็นพนักงาน\t\t\t11\n\n\tสวัสดิการและอื่นๆ\t\t\t\t13\n\nบริษัท เค.พี.เทรดดิ้ง กรุ๊ป จำกัด\n\nเป็นบริษัท ขายสายไฟและอุปกรณ์ มากมายหลายชนิดรวมทั้ง\n\nจัดทำการแปรรูปสายไฟตามที่ลูกค้าต้องการ สั่งแล้วคอยรับได้ทันที\n\n\tเราบริการจัดส่งสินค้าให้ลูกค้าได้รับของภายใน 2 วัน\n\n    ดังนั้น งานรับ Order จากลูกค้า, งานแปรรูปสินค้าและการจัดส่งสินค้าให้ทันตามที่ลูกค้าต้องการ จึงเป็น Core business ของบริษัทนี้\n\nบริษัทถือหลักการที่ว่า จะปฏิบัติตามกฎหมายคุ้มครองแรงงาน เพื่อให้ลูกจ้างทำงานด้วยความปลอดภัย มีสุขอนามัยที่ดี ได้รับค่าตอบแทนและสวัสดิการตามกฎหมายคุ้มครองแรงงานปัจจุบัน (ธ.ค. 2561)',
 '<figure>  \n![](figures/0)  \n<!-- FigureContent="Yuanta Securities" -->  \n</figure>  \nYUANTA ESG RATING  \n<figure>  \n![](figures/1)  \n<!-- FigureContent="AA" -->  \n</figure>',
 '# Thailand : Food & Beverage  \nCompany Update BUY (Upgrade)  \nTP upside (downside) +15.9%  \nClose Mar 29, 2024 Price (THB) 14.50 12M Target (THB) 16.80 Previous Target (THB) 16.80',
 "# What's new?  \nคาดกำไรปกติ 1Q67 ที่ 949 ลบ. (-25.3% QOQ, +7.3% YOY) มองเป็นระดับต่ำสุดของ ปี โดยกำไรชะลอลง QoQ จากธุรกิจ Frozen และ Pet Care ที่ลดลงตามฤดูกาล แต่ เติบโต YOY จากฐานต่ำ)  \nแนวโน้มกำไรปกติ 2Q67 คาดกลับมาเติบโต ทั้ง QOQ และ Yoy จากปริมาณขายที่เติบโต และบริษัทจะรับรู้ราคาต้นทุนวัตถุดิบที่ ลดลงได้มากขึ้นหนุน GPM",
 '## Our view  \nเรามีมุมมองบวกต่อแนวโน้มผลประกอบการ ของ TU ที่คาดจะเติบโตได้ทั้ง QoQ และ YOY ต่อเนื่องใน 2Q-3Q67 จากธุรกิจหลักที่ฟื้นตัว และไม่ต้องรับรู้ขาดทุนจาก Red Lobster  \nเราคงประมาณการกำไรปกติปี 2567 ที่ 5,965 ลบ. (+5.8% Yoy) คงราคาเป้าหมายที่ 16.80 บาท ราคาปัจจุบันมี Upside gain 15.9% เริ่ม น่าสนใจ จึงปรับคำแนะนำขึ้นเป็น "ซื้อ"  \nCompany profile: The Company engages in the manufacture and export of frozen and canned seafood. Its comprehensive business is completed with snack foods such as canned food, frozen food and snacks of various types especially seafood. Also, its businesses include packaging and publishing business, domestic market business, animal feed and development of shrimp species for sale business and commercial shrimp hatchery and nursery.  \nResearch Analysts:  \n<figure>  \n![](figures/2)  \n</figure>  \nKritawit Ratanakhanokchai  \n|||\n| - | - |\n| Tel. | +662 009 8064 |\n| E-Mail | Kritawit.r@yuanta.co.th |\n| ID | 124392 |  \n| Bloomberg code | TU TB |\n| - | - |\n| Market cap (THBmn) | 67,499.42 |\n| Corporate Gov.Rating | |\n| 6M avg. daily turnover (THBmn) | 296.43 |\n| Outstanding shares (mn) | 4,655.13 |\n| Free float (%) | 69.88% |\n| Major shareholders (%) | |\n| Thai NVDR Company Limited | 7.45 |\n| MR. THIRAPHONG CHANSIRI | 6.86 |\n| MITSUBISHI UFJ MORGAN STANLEY SECURITIES CO.,LTD. | 5.13 |',
 '# Financial outlook (THBmn)  \n| Year to Dec | 2022A | 2023A | 2024F | 2025F |\n| - | - | - | - | - |\n| Revenue | 155,586 | 136,153 | 142,010 | 153,083 |\n| EBITDA | 12,885 | 12,311 | 14,132 | 15,318 |\n| Core Profit | 7,201 | 5,640 | 5,965 | 6,910 |\n| Net Profit | 7,138 | -13,933 | 5,965 | 6,910 |\n| Core EPS | 1.51 | 1.21 | 1.28 | 1.48 |\n| Core EPS Growth (%) | -10.8 | -19.7 | 5.8 | 15.8 |\n| DPS | 0.84 | 0.54 | 0.73 | 0.85 |\n| Core P/E (x) | 9.6 | 12.0 | 11.3 | 9.8 |\n| P/BV(x) | 0.9 | 1.2 | 1.1 | 1.1 |\n| Div Yield (%) | 5.8 | 3.7 | 5.0 | 5.8 |\n| Net gearing (%) | 53.1 | 79.3 | 77.9 | 75.4 |\n| ROE (%) | 8.1 | -21.1 | 8.6 | 9.5 |  \nSource: Company, Yuanta Research  \n1 April 2024  \nValuation Methodology: SOTP; Seafood Business (PER 12.4x), Pet Care Business (Holding Discount 40%)',
 '# Thai Union Group (TU)  \n## คาดกำไร 1Q67 จะเป็นระดับต่ำสุดของปีนี้  \nคาดกำไรปกติ 1Q67 เติบโต Yoy ดีกว่าเดิมที่ประเมินว่าทำได้เพียงทรงตัว  \nเราคาดกำไรปกติ 1Q67 ของ TU ที่ 949 ลบ. (-25.3% QoQ, +7.3% Yoy) ดีกว่าเดิมที่เราประเมินว่าจะทำ ได้เพียงทรงตัว Yoy และอาจเป็นระดับกำไรต่ำสุดของปี โดยเราคาดรายได้ที่ 3.33 หมื่นลบ. (-6.2% QoQ, +2.1% Yoy) การชะลอลง QoQ มาจากรายได้ที่ลดลงในธุรกิจ Frozen และ Petcare จาก Demand ที่ ชะลอตัวลงในตลาดสหรัฐฯ หลังมีการสั่งซื้อไปมากใน 4Q67 กระทบต่อเนื่องต่อ GPM ให้ลดลง (ปริมาณ ขายลดลงส่งผลให้ต้นทุนต่อหน่วยสูงขึ้น) ส่วนรายได้ธุรกิจ Ambient เห็นการฟื้นตัวได้เล็กน้อย QoQ ขณะที่ต้นทุนราคาปลาทูน่าที่ปรับลดลงมาเฉลี่ยอยู่ที่ 1,350 USD/ตัน จากเฉลี่ย 1,500 USD/ตัน ใน 4Q66 จะยังไม่ได้ส่งผลบวกต่อ GPM ของ TU เนื่องจากบริษัทต้องปรับลดราคาขายสินค้ากลุ่ม OEM ตามราคา ต้นทุนทูน่า ทำให้เราคาด GPM อยู่ที่ 16.8% ลดลงจาก 17.8% ใน 4Q66 แต่เทียบ Yoy ดีขึ้นจาก 15.1% ใน 1Q66 นอกจากนี้ TU จะไม่ต้องรับรู้ส่วนแบ่งกำไร/ขาดทุนจาก Red Lobster เป็นไตรมาสแรก  \n### คาดกำไรปกติกลับมาเติบโตได้ทั้ง QoQ และ Yoy ได้ใน 2Q-3Q67  \nเราประเมินว่ากำไรของ TU จะกลับมาเติบโตทั้ง QoQ และ Yoy อีกครั้งได้ใน 2Q67 และต่อเนื่องถึง 3Q67 ที่เป็นช่วง High Season ของธุรกิจ จากการฟื้นตัวของรายได้ และ GPM แบบ QOQ อย่างค่อยเป็นค่อยไป เนื่องจากบริษัทจะเริ่มรับรู้ราคาต้นทุนเฉลี่ยที่ปรับลงได้มากขึ้น (ปกติจะมี Lag time ราว 2 - 3 เดือน) ขณะที่การปรับราคาสินค้ากลุ่ม OEM ลงใกล้สิ้นสุดแล้วตามราคาปลาทูน่าที่คาดเริ่มมี Downside จำกัด ขณะที่ฝั่ง Demand จากลูกค้าฟื้นตัวหลังมีการชะลอไปใน 1Q67 และคาดกำไรปกติจะเติบโตเด่นเทียบ YOY จากฐานต่ำในปีก่อนที่ลูกค้าตลาดสหรัฐฯ และยุโรปอยู่ในช่วงระบายสต็อกสินค้า นอกจากนี้จะไม่ต้อง รับรู้ส่วนแบ่งขาดทุนจาก Red Lobster เหมือนในปีก่อน',
 '## คงราคาเหมาะสมที่ 16.80 บาท แต่ปรับคำแนะนำขึ้นเป็น "ซื้อ"  \nเราคงมุมมองบวกต่อแนวโน้มผลประกอบการในปี 2567 ของ TU ที่คาดจะฟื้นตัวอย่างค่อยเป็นค่อยไป คาดกำไรปกติที่ 5,965 ลบ. (+5.8% Yoy) เราคงราคาเหมาะสมที่ 16.80 บาท (อิงวิธี SOTP) มี Upside gain 15.9% และราคาปัจจุบันซื้อขายบน PER67 เพียง 12 เท่า มองว่าเริ่มน่าดึงดูด จึงปรับคำแนะนำขึ้น จาก TRADING เป็น "ซื้อ" เหมาะสำหรับการลงทุนระยะยาว  \nบริษัทหลักทรัพย์ หยวนต้า (ประเทศไทย) จำกัด อาจเป็นผู้ออกและเสนอขายใบสำคัญแสดงสิทธิ์อนุพันธ์ (Derivative Warrants) ที่อ้างอิงหลักทรัพย์ใน SET100 และ SET50 index และอาจเป็นผู้ดูแลสภาพคล่อง (Market Maker) โดยบริษัทฯ อาจจัดทำบทวิเคราะห์ของหลักทรัพย์อ้างอิงดังกล่าว ดังนั้น นักลงทุนควรศึกษารายละเอียดในหนังสือชี้ชวน ของใบสำคัญแสดงสิทธิอนุพันธ์ดังกล่าวก่อนการตัดสินใจลงทุน\n<figure>  \n![](figures/3)  \n<!-- FigureContent="Yuanta Securities" -->  \n</figure>  \nWe Create Fortune\n===  \nFigure 1: Quarterly Earnings Preview  \n| (Unit : THB mn) | 1Q24F | 4Q23 | %QoQ | 1Q23 | %YoY |\n| - | - | - | - | - | - |\n| Core revenue | 33,337 | 35,529 | -6.2% | 32,652 | 2.1% |\n| COGS | (27,752) | (29,201) | -5.0% | (27,735) | 0.1% |\n| Gross profit | 5,585 | 6,327 | -11.7% | 4,917 | 13.6% |\n| SG&A | (4,048) | (4,173) | -3.0% | (4,121) | -1.8% |\n| Operating Profit | 1,884 | 2,061 | -8.6% | 1,450 | 29.9% |\n| Depre & amortization | 1,190 | 1,073 | 10.9% | 1,049 | 13.5% |\n| Operating EBITDA | 3,452 | 3,134 | n.a. | 2,499 | n.a. |\n| Other incomes (expenses) | 126 | 304 | n.a. | 424 | n.a. |\n| Financial cost | (659) | (630) | 4.6% | (532) | 23.8% |\n| Share of profit (loss) from associates | 220 | (398) | n.a. | 231 | n.a. |\n| Pre-tax profit | 1,225 | 1,431 | -14.4% | 917 | 33.5% |\n| Taxes | (66) | 40 | -264.9% | 242 | -127.4% |\n| Minorities | (283) | (244) | 16.1% | (137) | 105.9% |\n| Net profit | 875 | (18,406) | n.a. | 1,022 | n.a. |\n| Extraordinary items | (74) | (19,676) | n.a. | 137 | n.a. |\n| Core profit | 949 | 1,270 | -25.3% | 884 | 7.3% |\n| Core EPS (THB) | 0.20 | 0.27 | -25.3% | 0.19 | 7.3% |\n| Margins (%) | | | Dif QoQ | | Dif YoY |\n| Gross profit margin | 16.8 | 17.8 | -1.06 | 15.1 | 1.70 |\n| Operating margin | 5.6 | 5.8 | -0.15 | 4.4 | 1.21 |\n| EBITDA margin | 10.4 | 8.8 | 1.53 | 7.7 | 2.70 |\n| Norm profit margin | 2.8 | 3.6 | -0.73 | 2.7 | 0.14 |',
 '## Source: Company, Yuanta Research  \nFigure 2: Skipjack Tuna price  \n<figure>  \n![](figures/4)  \n<!-- FigureContent="Skipjack Tuna Price Unit : USD/Ton 2,200 2,000 1,980 2,000 2,000 2,000 1,900 1,800 1,800 1,800 1,680 1,700 1,600 1,600 1,500 1400 1,450 1,400 1300 1,200 1,000 800 Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec -2020 -- 2021 -2022 -- 2023 -2024" -->  \n<figcaption>  \nSource: Company, Yuanta Research  \n</figcaption>  \n</figure>  \n<!-- PageFooter="Thailand" -->  \n<!-- PageNumber="Page 2 of 8" -->'
 ]
        embedding = await client.batch_get_embedding(text)
        print(embedding)

    asyncio.run(main())